@@ -163,7 +163,7 @@ namespace
                         m_p(p), m_f(f) {}
                     virtual ~callback_function_c() throw() {}
 
-                    virtual void operator () (int const command,
+                    virtual void operator () (int const request_type,
                                               char const * const name,
                                               char const * const pattern,
                                               void const * const request_info,
@@ -176,8 +176,7 @@ namespace
                                               char const * const pid,
                                               uint32_t const pid_size)
                     {
-                        m_f(m_p,
-                            command,
+                        m_f(request_type,
                             name,
                             pattern,
                             request_info,
@@ -188,7 +187,9 @@ namespace
                             priority,
                             trans_id,
                             pid,
-                            pid_size);
+                            pid_size,
+                            m_p->state,
+                            m_p);
                     }
                 private:
                     cloudi_instance_t * m_p;
@@ -231,7 +232,7 @@ namespace
                 return null_response;
             }
 
-            void operator () (int const command,
+            void operator () (int const request_type,
                               char const * const name,
                               char const * const pattern,
                               void const * const request_info,
@@ -244,7 +245,7 @@ namespace
                               char const * const pid,
                               uint32_t const pid_size) const
             {
-                (*m_function)(command,
+                (*m_function)(request_type,
                               name,
                               pattern,
                               request_info,
@@ -558,7 +559,8 @@ static int poll_request(cloudi_instance_t * p,
                         int external);
 
 int cloudi_initialize(cloudi_instance_t * p,
-                      unsigned int const thread_index)
+                      unsigned int const thread_index,
+                      void * state)
 {
     if (p == 0)
         return cloudi_out_of_memory;
@@ -569,6 +571,7 @@ int cloudi_initialize(cloudi_instance_t * p,
     if (buffer_size_p == 0)
         return cloudi_invalid_input;
     ::memset(p, 0, sizeof(cloudi_instance_t));
+    p->state = state;
     uint32_t const buffer_size = ::atoi(buffer_size_p);
     if (::strcmp(protocol, "tcp") == 0)
     {
@@ -632,10 +635,10 @@ int cloudi_initialize(cloudi_instance_t * p,
     return result;
 }
 
-void cloudi_destroy(cloudi_instance_t * p)
+void * cloudi_destroy(cloudi_instance_t * p)
 {
     if (p == 0)
-        return;
+        return 0;
     if (p->fd_in != 0)
     {
         ::close(p->fd_in);
@@ -649,7 +652,9 @@ void cloudi_destroy(cloudi_instance_t * p)
         delete reinterpret_cast<timer *>(p->request_timer);
         if (p->prefix)
             delete [] p->prefix;
+        return p->state;
     }
+    return 0;
 }
 
 int cloudi_initialize_thread_count(unsigned int * const thread_count)
@@ -950,7 +955,7 @@ static int cloudi_forward_(cloudi_instance_t * p,
 }
 
 int cloudi_forward(cloudi_instance_t * p,
-                   int const command,
+                   int const request_type,
                    char const * const name,
                    void const * const request_info,
                    uint32_t const request_info_size,
@@ -963,7 +968,7 @@ int cloudi_forward(cloudi_instance_t * p,
                    uint32_t const pid_size)
 {
     int result;
-    if (command == CLOUDI_ASYNC)
+    if (request_type == CLOUDI_ASYNC)
     {
         result = cloudi_forward_(p,
                                  "forward_async", name,
@@ -976,7 +981,7 @@ int cloudi_forward(cloudi_instance_t * p,
             throw CloudI::API::forward_async_exception();
         }
     }
-    else if (command == CLOUDI_SYNC)
+    else if (request_type == CLOUDI_SYNC)
     {
         result = cloudi_forward_(p,
                                  "forward_sync", name,
@@ -1114,7 +1119,7 @@ static int cloudi_return_(cloudi_instance_t * p,
 }
 
 int cloudi_return(cloudi_instance_t * p,
-                  int const command,
+                  int const request_type,
                   char const * const name,
                   char const * const pattern,
                   void const * const response_info,
@@ -1127,7 +1132,7 @@ int cloudi_return(cloudi_instance_t * p,
                   uint32_t const pid_size)
 {
     int result;
-    if (command == CLOUDI_ASYNC)
+    if (request_type == CLOUDI_ASYNC)
     {
         result = cloudi_return_(p,
                                 "return_async", name, pattern,
@@ -1139,7 +1144,7 @@ int cloudi_return(cloudi_instance_t * p,
             throw CloudI::API::return_async_exception();
         }
     }
-    else if (command == CLOUDI_SYNC)
+    else if (request_type == CLOUDI_SYNC)
     {
         result = cloudi_return_(p,
                                 "return_sync", name, pattern,
@@ -1479,6 +1484,7 @@ static bool handle_events(cloudi_instance_t * p,
                           uint32_t command = 0)
 {
     buffer_t & buffer_recv = *reinterpret_cast<buffer_t *>(p->buffer_recv);
+    timer & request_timer = *reinterpret_cast<timer *>(p->request_timer);
     if (command == 0)
     {
         if (index > p->buffer_recv_index)
@@ -1504,6 +1510,16 @@ static bool handle_events(cloudi_instance_t * p,
             case MESSAGE_REINIT:
             {
                 store_incoming_uint32(buffer_recv, index, p->process_count);
+                store_incoming_uint32(buffer_recv, index, p->timeout_async);
+                store_incoming_uint32(buffer_recv, index, p->timeout_sync);
+                store_incoming_int8(buffer_recv, index, p->priority_default);
+                store_incoming_uint8(buffer_recv, index,
+                                     p->request_timeout_adjustment);
+                if (p->request_timeout_adjustment)
+                {
+                    request_timer.restart();
+                    p->request_timeout = 0;
+                }
                 break;
             }
             case MESSAGE_KEEPALIVE:
@@ -1551,6 +1567,7 @@ static int poll_request(cloudi_instance_t * p,
 
     buffer_t & buffer_recv = *reinterpret_cast<buffer_t *>(p->buffer_recv);
     buffer_t & buffer_call = *reinterpret_cast<buffer_t *>(p->buffer_call);
+    timer & request_timer = *reinterpret_cast<timer *>(p->request_timer);
 
     timer & poll_timer = *reinterpret_cast<timer *>(p->poll_timer);
     if (timeout > 0)
@@ -1718,6 +1735,16 @@ static int poll_request(cloudi_instance_t * p,
             case MESSAGE_REINIT:
             {
                 store_incoming_uint32(buffer_recv, index, p->process_count);
+                store_incoming_uint32(buffer_recv, index, p->timeout_async);
+                store_incoming_uint32(buffer_recv, index, p->timeout_sync);
+                store_incoming_int8(buffer_recv, index, p->priority_default);
+                store_incoming_uint8(buffer_recv, index,
+                                     p->request_timeout_adjustment);
+                if (p->request_timeout_adjustment)
+                {
+                    request_timer.restart();
+                    p->request_timeout = 0;
+                }
                 if (index == p->buffer_recv_index)
                 {
                     p->buffer_recv_index = 0;
@@ -1823,17 +1850,6 @@ static void text_key_value_destroy(char const ** p)
 
 // CloudI helper functions
 
-char const ** cloudi_request_http_qs_parse(void const * const request,
-                                           uint32_t const request_size)
-{
-    return text_key_value_parse(request, request_size);
-}
-
-void cloudi_request_http_qs_destroy(char const ** p)
-{
-    text_key_value_destroy(p);
-}
-
 char const ** cloudi_info_key_value_parse(void const * const message_info,
                                           uint32_t const message_info_size)
 {
@@ -1856,7 +1872,7 @@ API::API(unsigned int const thread_index) :
     m_count(new int)
 {
     (*m_count) = 1;
-    int const result = cloudi_initialize(m_api, thread_index);
+    int const result = cloudi_initialize(m_api, thread_index, 0);
     if (result != return_value::success)
         throw invalid_input_exception(result);
 }
@@ -2036,7 +2052,7 @@ uint32_t API::get_subscribe_count() const
     return m_api->subscribe_count;
 }
 
-int API::forward_(int const command,
+int API::forward_(int const request_type,
                   char const * const name,
                   void const * const request_info,
                   uint32_t const request_info_size,
@@ -2049,7 +2065,7 @@ int API::forward_(int const command,
                   uint32_t const pid_size) const
 {
     return cloudi_forward(m_api,
-                          command,
+                          request_type,
                           name,
                           request_info,
                           request_info_size,
@@ -2110,7 +2126,7 @@ int API::forward_sync(char const * const name,
                                pid_size);
 }
 
-int API::return_(int const command,
+int API::return_(int const request_type,
                  char const * const name,
                  char const * const pattern,
                  void const * const response_info,
@@ -2123,7 +2139,7 @@ int API::return_(int const command,
                  uint32_t const pid_size) const
 {
     return cloudi_return(m_api,
-                         command,
+                         request_type,
                          name,
                          pattern,
                          response_info,
@@ -2301,18 +2317,6 @@ int API::poll(int timeout) const
                        timeout);
 }
 
-char const ** API::request_http_qs_parse(void const * const request,
-                                         uint32_t const request_size) const
-{
-    return cloudi_request_http_qs_parse(request,
-                                        request_size);
-}
-
-void API::request_http_qs_destroy(char const ** p) const
-{
-    cloudi_request_http_qs_destroy(p);
-}
-
 char const ** API::info_key_value_parse(void const * const message_info,
                                         uint32_t const message_info_size) const
 {
