@@ -3,7 +3,7 @@
 //
 // BSD LICENSE
 // 
-// Copyright (c) 2014-2016, Michael Truog <mjtruog at gmail dot com>
+// Copyright (c) 2014-2017, Michael Truog <mjtruog at gmail dot com>
 // All rights reserved.
 // 
 // Redistribution and use in source and binary forms, with or without
@@ -259,7 +259,7 @@ CloudI.API = function API (thread_index, callback) {
             else if (data.length < API._poll_data_size) {
                 API._poll_data = data;
             }
-            else if (data.length > API._poll_data_size) {
+            else if (data.length > API._poll_data_size + 4) {
                 var next_data = data.slice(API._poll_data_size);
                 data = data.slice(0, API._poll_data_size);
                 API._poll_data = next_data;
@@ -404,10 +404,11 @@ CloudI.API.prototype.mcast_async = function (name, request, callback,
     });
 };
 
-CloudI.API.prototype.forward_ = function (command, name, request_info, request,
+CloudI.API.prototype.forward_ = function (request_type, name,
+                                          request_info, request,
                                           timeout, priority, trans_id, pid) {
     var API = this;
-    switch (command) {
+    switch (request_type) {
         case API.ASYNC:
             API.forward_async(name, request_info, request,
                               timeout, priority, trans_id, pid);
@@ -471,11 +472,11 @@ CloudI.API.prototype.forward_sync = function (name, request_info, request,
     });
 };
 
-CloudI.API.prototype.return_ = function (command, name, pattern,
+CloudI.API.prototype.return_ = function (request_type, name, pattern,
                                          response_info, response,
                                          timeout, trans_id, pid) {
     var API = this;
-    switch (command) {
+    switch (request_type) {
         case API.ASYNC:
             API.return_async(name, pattern, response_info, response,
                              timeout, trans_id, pid);
@@ -591,7 +592,7 @@ CloudI.API.prototype.timeout_terminate = function () {
     return this._timeout_terminate;
 };
 
-CloudI.API.prototype._null_response = function (command, name, pattern,
+CloudI.API.prototype._null_response = function (request_type, name, pattern,
                                                 request_info, request,
                                                 timeout, priority,
                                                 trans_id, pid) {
@@ -795,6 +796,18 @@ CloudI.API.prototype._handle_events = function (data, data_size, i, command) {
             case MESSAGE_REINIT:
                 API._process_count = unpackUint32(i, data);
                 i += 4;
+                API._timeout_async = unpackUint32(i, data);
+                i += 4;
+                API._timeout_sync = unpackUint32(i, data);
+                i += 4;
+                API._priority_default = unpackInt8(i, data);
+                i += 1;
+                API._request_timeout_adjustment = unpackUint8(i, data);
+                i += 1;
+                if (API._request_timeout_adjustment) {
+                    API._request_timer = new Date().getTime();
+                    API._request_timeout = 0;
+                }
                 break;
             case MESSAGE_KEEPALIVE:
                 API._send(new Erlang.OtpErlangAtom('keepalive'));
@@ -853,10 +866,8 @@ CloudI.API.prototype._poll_request = function (data) {
                 if (i != data_size) {
                     API._handle_events(data, data_size, i);
                 }
-                process.nextTick(function () {
-                    API._poll_callback(API);
-                    API._poll_callback = undefined;
-                });
+                API._poll_callback(API);
+                API._poll_callback = undefined;
                 return;
             case MESSAGE_SEND_ASYNC:
             case MESSAGE_SEND_SYNC:
@@ -902,12 +913,10 @@ CloudI.API.prototype._poll_request = function (data) {
                         process.exit(1);
                     }
                     else {
-                        process.nextTick(function () {
-                            API._callback(command, name, pattern,
-                                          request_info, request,
-                                          request_timeout, priority,
-                                          trans_id, pid_object);
-                        });
+                        API._callback(command, name, pattern,
+                                      request_info, request,
+                                      request_timeout, priority,
+                                      trans_id, pid_object);
                     }
                 });
                 return;
@@ -929,10 +938,8 @@ CloudI.API.prototype._poll_request = function (data) {
                 if (i != data_size) {
                     API._handle_events(data, data_size, i);
                 }
-                process.nextTick(function () {
-                    API._poll_callback(response_info, response, trans_id);
-                    API._poll_callback = undefined;
-                });
+                API._poll_callback(response_info, response, trans_id);
+                API._poll_callback = undefined;
                 return;
             case MESSAGE_RETURN_ASYNC:
                 var j = i + 16;
@@ -941,10 +948,8 @@ CloudI.API.prototype._poll_request = function (data) {
                 if (i != data_size) {
                     API._handle_events(data, data_size, i);
                 }
-                process.nextTick(function () {
-                    API._poll_callback(trans_id);
-                    API._poll_callback = undefined;
-                });
+                API._poll_callback(trans_id);
+                API._poll_callback = undefined;
                 return;
             case MESSAGE_RETURNS_ASYNC:
                 var trans_id_count = unpackUint32(i, data);
@@ -961,10 +966,8 @@ CloudI.API.prototype._poll_request = function (data) {
                 if (i != data_size) {
                     API._handle_events(data, data_size, i);
                 }
-                process.nextTick(function () {
-                    API._poll_callback(trans_ids);
-                    API._poll_callback = undefined;
-                });
+                API._poll_callback(trans_ids);
+                API._poll_callback = undefined;
                 return;
             case MESSAGE_SUBSCRIBE_COUNT:
                 var count = unpackUint32(i, data);
@@ -972,10 +975,8 @@ CloudI.API.prototype._poll_request = function (data) {
                 if (i != data_size) {
                     API._handle_events(data, data_size, i);
                 }
-                process.nextTick(function () {
-                    API._poll_callback(count);
-                    API._poll_callback = undefined;
-                });
+                API._poll_callback(count);
+                API._poll_callback = undefined;
                 return;
             case MESSAGE_TERM:
                 API._handle_events(data, data_size, i, command);
@@ -983,6 +984,18 @@ CloudI.API.prototype._poll_request = function (data) {
             case MESSAGE_REINIT:
                 API._process_count = unpackUint32(i, data);
                 i += 4;
+                API._timeout_async = unpackUint32(i, data);
+                i += 4;
+                API._timeout_sync = unpackUint32(i, data);
+                i += 4;
+                API._priority_default = unpackInt8(i, data);
+                i += 1;
+                API._request_timeout_adjustment = unpackUint8(i, data);
+                i += 1;
+                if (API._request_timeout_adjustment) {
+                    API._request_timer = new Date().getTime();
+                    API._request_timeout = 0;
+                }
                 if (i == data_size) {
                     return;
                 }
@@ -1101,10 +1114,6 @@ CloudI.API.prototype._text_key_value_parse = function (text) {
     return result;
 };
 
-CloudI.API.prototype.request_http_qs_parse = function (request) {
-    return CloudI.API.prototype._text_key_value_parse(request);
-};
-
 CloudI.API.prototype.info_key_value_parse = function (message_info) {
     return CloudI.API.prototype._text_key_value_parse(message_info);
 };
@@ -1128,9 +1137,7 @@ CloudI.API.prototype._send = function (terms) {
         if (API._use_header) {
             data = Buffer.concat([packUint32big(data.length), data]);
         }
-        process.nextTick(function () {
-            API._s_out.write(data, 'binary');
-        });
+        API._s_out.write(data, 'binary');
     });
 };
 
