@@ -2,13 +2,13 @@
 # ex: set ft=perl fenc=utf-8 sts=4 ts=4 sw=4 et nomod:
 #
 # BSD LICENSE
-# 
+#
 # Copyright (c) 2014-2017, Michael Truog <mjtruog at gmail dot com>
 # All rights reserved.
-# 
+#
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
-# 
+#
 #     * Redistributions of source code must retain the above copyright
 #       notice, this list of conditions and the following disclaimer.
 #     * Redistributions in binary form must reproduce the above copyright
@@ -21,7 +21,7 @@
 #     * The name of the author may not be used to endorse or promote
 #       products derived from this software without specific prior
 #       written permission
-# 
+#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 # CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 # INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
@@ -43,7 +43,7 @@ use strict;
 use warnings;
 use 5.010;
 
-$CloudI::API::VERSION = '1.60';
+$CloudI::API::VERSION = '1.70';
 
 use POSIX qw(getenv);
 use IO::Handle;
@@ -310,14 +310,14 @@ sub mcast_async
 sub forward_
 {
     my $self = shift;
-    my ($command, $name, $request_info, $request,
+    my ($request_type, $name, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
-    if ($command == ASYNC)
+    if ($request_type == ASYNC)
     {
         $self->forward_async($name, $request_info, $request,
                              $timeout, $priority, $trans_id, $pid);
     }
-    elsif ($command == SYNC)
+    elsif ($request_type == SYNC)
     {
         $self->forward_sync($name, $request_info, $request,
                             $timeout, $priority, $trans_id, $pid);
@@ -387,14 +387,14 @@ sub forward_sync
 sub return_
 {
     my $self = shift;
-    my ($command, $name, $pattern, $response_info, $response,
+    my ($request_type, $name, $pattern, $response_info, $response,
         $timeout, $trans_id, $pid) = @_;
-    if ($command == ASYNC)
+    if ($request_type == ASYNC)
     {
         $self->return_async($name, $pattern, $response_info, $response,
                             $timeout, $trans_id, $pid);
     }
-    elsif ($command == SYNC)
+    elsif ($request_type == SYNC)
     {
         $self->return_sync($name, $pattern, $response_info, $response,
                            $timeout, $trans_id, $pid);
@@ -547,7 +547,7 @@ sub timeout_terminate
 sub _null_response
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request,
+    my ($request_type, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
     return '';
 }
@@ -759,8 +759,19 @@ sub _handle_events
         }
         elsif ($command == MESSAGE_REINIT)
         {
-            $i += $j; $j = 4;
-            ($self->{_process_count}) = unpack('L', substr($data, $i, $j));
+            $i += $j; $j = 4 + 4 + 4 + 1 + 1;
+            ($self->{_process_count},
+             $self->{_timeout_async},
+             $self->{_timeout_sync},
+             $self->{_priority_default},
+             $self->{_request_timeout_adjustment}) = unpack("L3 c C",
+                                                            substr($data,
+                                                                   $i, $j));
+            if ($self->{_request_timeout_adjustment})
+            {
+                $self->{_request_timer} = _milliseconds();
+                $self->{_request_timeout} = 0;
+            }
             $i += $j;
         }
         elsif ($command == MESSAGE_KEEPALIVE)
@@ -983,12 +994,23 @@ sub _poll_request
         }
         elsif ($command == MESSAGE_REINIT)
         {
-            $i += $j; $j = 4;
-            ($self->{_process_count}) = unpack('L', substr($data, $i, $j));
+            $i += $j; $j = 4 + 4 + 4 + 1 + 1;
+            ($self->{_process_count},
+             $self->{_timeout_async},
+             $self->{_timeout_sync},
+             $self->{_priority_default},
+             $self->{_request_timeout_adjustment}) = unpack("L3 c C",
+                                                            substr($data,
+                                                                   $i, $j));
+            if ($self->{_request_timeout_adjustment})
+            {
+                $self->{_request_timer} = _milliseconds();
+                $self->{_request_timeout} = 0;
+            }
             $i += $j; $j = 4;
             if ($i == $data_size)
             {
-                1;
+                $data = '';
             }
             elsif ($i < $data_size)
             {
@@ -1006,7 +1028,7 @@ sub _poll_request
             $i += $j; $j = 4;
             if ($i == $data_size)
             {
-                1;
+                $data = '';
             }
             elsif ($i < $data_size)
             {
@@ -1061,7 +1083,7 @@ sub _poll_request
         {
             return 1;
         }
-    
+
         $data = $self->_recv($data);
         $data_size = length($data);
         if ($data_size == 0)
@@ -1118,13 +1140,6 @@ sub _text_key_value_parse
     return %result;
 }
 
-sub request_http_qs_parse
-{
-    my $self = shift;
-    my ($request) = @_;
-    return $self->_text_key_value_parse($request);
-}
-
 sub info_key_value_parse
 {
     my $self = shift;
@@ -1146,15 +1161,16 @@ sub _send
 sub _recv
 {
     my $self = shift;
-    my ($data) = @_;
+    my ($data_old) = @_;
+    my $data = '';
     my $fragment = '';
     my $read;
-    my $i = length($data);
     if ($self->{_use_header})
     {
+        my $i = 0;
         while ($i < 4)
         {
-            $read = sysread($self->{_s}, $fragment, $self->{_size});
+            $read = sysread($self->{_s}, $fragment, 4 - $i);
             if (! defined($read) || $read == 0)
             {
                 die CloudI::MessageDecodingException->new();
@@ -1162,12 +1178,13 @@ sub _recv
             $i += $read;
             $data .= $fragment;
         }
-        my ($total) = unpack('N', substr($data, 0, 4));
-        $i -= 4;
-        $data = substr($data, 4);
+        my ($total) = unpack('N', $data);
+        $i = 0;
+        $data = $data_old;
         while ($i < $total)
         {
-            $read = sysread($self->{_s}, $fragment, $self->{_size});
+            $read = sysread($self->{_s}, $fragment,
+                            _min($total - $i, $self->{_size}));
             if (! defined($read) || $read == 0)
             {
                 die CloudI::MessageDecodingException->new();
@@ -1178,6 +1195,8 @@ sub _recv
     }
     else
     {
+        $data = $data_old;
+        my $i = length($data);
         my $ready = 1;
         while ($ready)
         {
@@ -1214,6 +1233,19 @@ sub _milliseconds
     return $seconds * 1000 + $microseconds / 1000;
 }
 
+sub _min
+{
+    my ($min, @args) = @_;
+    for my $v (@args)
+    {
+        if ($v < $min)
+        {
+            $min = $v;
+        }
+    }
+    return $min;
+}
+
 sub _max
 {
     my ($max, @args) = @_;
