@@ -3,13 +3,13 @@
 # ex: set ft=python fenc=utf-8 sts=4 ts=4 sw=4 et nomod:
 #
 # BSD LICENSE
-# 
-# Copyright (c) 2011-2016, Michael Truog <mjtruog at gmail dot com>
+#
+# Copyright (c) 2011-2017, Michael Truog <mjtruog at gmail dot com>
 # All rights reserved.
-# 
+#
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
-# 
+#
 #     * Redistributions of source code must retain the above copyright
 #       notice, this list of conditions and the following disclaimer.
 #     * Redistributions in binary form must reproduce the above copyright
@@ -22,7 +22,7 @@
 #     * The name of the author may not be used to endorse or promote
 #       products derived from this software without specific prior
 #       written permission
-# 
+#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 # CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 # INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
@@ -185,13 +185,13 @@ class API(object):
                                     timeout, priority)))
         return self.__poll_request(None, False)
 
-    def forward_(self, command, name, request_info, request,
+    def forward_(self, request_type, name, request_info, request,
                  timeout, priority, trans_id, pid):
-        if command == API.ASYNC:
+        if request_type == API.ASYNC:
             self.forward_async(name,
                                request_info, request,
                                timeout, priority, trans_id, pid)
-        elif command == API.SYNC:
+        elif request_type == API.SYNC:
             self.forward_sync(name, pattern,
                               request_info, request,
                               timeout, priority, trans_id, pid)
@@ -232,13 +232,13 @@ class API(object):
                                     OtpErlangBinary(trans_id), pid)))
         raise forward_sync_exception()
 
-    def return_(self, command, name, pattern, response_info, response,
+    def return_(self, request_type, name, pattern, response_info, response,
                 timeout, trans_id, pid):
-        if command == API.ASYNC:
+        if request_type == API.ASYNC:
             self.return_async(name, pattern,
                               response_info, response,
                               timeout, trans_id, pid)
-        elif command == API.SYNC:
+        elif request_type == API.SYNC:
             self.return_sync(name, pattern,
                              response_info, response,
                              timeout, trans_id, pid)
@@ -319,7 +319,8 @@ class API(object):
     def timeout_terminate(self):
         return self.__timeout_terminate
 
-    def __null_response(self, command, name, pattern, request_info, request,
+    def __null_response(self, request_type, name, pattern,
+                        request_info, request,
                         timeout, priority, trans_id, pid):
         return b''
 
@@ -443,8 +444,19 @@ class API(object):
                 else:
                     raise terminate_exception(self.__timeout_terminate)
             elif command == _MESSAGE_REINIT:
-                i, j = j, j + 4
-                self.__process_count = struct.unpack(b'=I', data[i:j])[0]
+                i, j = j, j + 4 + 4 + 4 + 1 + 1
+                (self.__process_count,
+                 self.__timeout_async, self.__timeout_sync,
+                 self.__priority_default,
+                 request_timeout_adjustment) = struct.unpack(
+                    b'=IIIbB', data[i:j]
+                )
+                self.__request_timeout_adjustment = bool(
+                    request_timeout_adjustment
+                )
+                if self.__request_timeout_adjustment:
+                    self.__request_timer = default_timer()
+                    self.__request_timeout = 0
             elif command == _MESSAGE_KEEPALIVE:
                 self.__send(term_to_binary(OtpErlangAtom(b'keepalive')))
             else:
@@ -592,10 +604,21 @@ class API(object):
                     return False
                 assert False
             elif command == _MESSAGE_REINIT:
-                i, j = j, j + 4
-                self.__process_count = struct.unpack(b'=I', data[i:j])[0]
+                i, j = j, j + 4 + 4 + 4 + 1 + 1
+                (self.__process_count,
+                 self.__timeout_async, self.__timeout_sync,
+                 self.__priority_default,
+                 request_timeout_adjustment) = struct.unpack(
+                    b'=IIIbB', data[i:j]
+                )
+                self.__request_timeout_adjustment = bool(
+                    request_timeout_adjustment
+                )
+                if self.__request_timeout_adjustment:
+                    self.__request_timer = default_timer()
+                    self.__request_timeout = 0
                 if j == data_size:
-                    pass
+                    data = b''
                 elif j < data_size:
                     i, j = j, j + 4
                     continue
@@ -604,7 +627,7 @@ class API(object):
             elif command == _MESSAGE_KEEPALIVE:
                 self.__send(term_to_binary(OtpErlangAtom(b'keepalive')))
                 if j == data_size:
-                    pass
+                    data = b''
                 elif j < data_size:
                     i, j = j, j + 4
                     continue
@@ -633,7 +656,7 @@ class API(object):
                 return False
             if len(IN) == 0:
                 return True
-    
+
             data = self.__recv(data)
             data_size = len(data)
             if data_size == 0:
@@ -657,9 +680,6 @@ class API(object):
                 result[key] = [current, data[i + 1]]
         return result
 
-    def request_http_qs_parse(self, request):
-        return self.__text_key_value_parse(request)
-
     def info_key_value_parse(self, message_info):
         return self.__text_key_value_parse(message_info)
 
@@ -668,17 +688,23 @@ class API(object):
             data = struct.pack(b'>I', len(data)) + data
         self.__s.sendall(data)
 
-    def __recv(self, data):
+    def __recv(self, data_old):
+        data = ''
         if self.__use_header:
-            while len(data) < 4:
-                fragment = self.__s.recv(self.__size)
+            i = 0
+            while i < 4:
+                fragment = self.__s.recv(4 - i)
                 data += fragment
-            total = struct.unpack(b'>I', data[:4])[0]
-            data = data[4:]
-            while len(data) < total:
-                fragment = self.__s.recv(self.__size)
+                i += len(fragment)
+            total = struct.unpack(b'>I', data)[0]
+            data = data_old
+            i = 0
+            while i < total:
+                fragment = self.__s.recv(min(total - i, self.__size))
                 data += fragment
+                i += len(fragment)
         else:
+            data = data_old
             ready = True
             while ready == True:
                 fragment = self.__s.recv(self.__size)
@@ -738,14 +764,14 @@ if sys.stderr.__class__.__name__ != '_unbuffered':
             else:
                 import codecs
                 self.__stream = codecs.getwriter('UTF-8')(stream)
-    
+
         def write(self, data):
             self.__stream.write(data)
             self.__stream.flush()
-    
+
         def __getattr__(self, attr):
             return getattr(self.__stream, attr)
-    
+
     sys.stdout = _unbuffered(sys.stdout)
     sys.stderr = _unbuffered(sys.stderr)
 
