@@ -3,13 +3,13 @@
 # ex: set ft=ruby fenc=utf-8 sts=4 ts=4 sw=4 et nomod:
 #
 # BSD LICENSE
-# 
-# Copyright (c) 2011-2016, Michael Truog <mjtruog at gmail dot com>
+#
+# Copyright (c) 2011-2017, Michael Truog <mjtruog at gmail dot com>
 # All rights reserved.
-# 
+#
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
-# 
+#
 #     * Redistributions of source code must retain the above copyright
 #       notice, this list of conditions and the following disclaimer.
 #     * Redistributions in binary form must reproduce the above copyright
@@ -22,7 +22,7 @@
 #     * The name of the author may not be used to endorse or promote
 #       products derived from this software without specific prior
 #       written permission
-# 
+#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 # CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 # INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
@@ -176,9 +176,9 @@ module CloudI
             return poll_request(nil, false)
         end
 
-        def forward_(command, name, request_info, request,
+        def forward_(request_type, name, request_info, request,
                      timeout, priority, trans_id, pid)
-            case command
+            case request_type
             when ASYNC
                 forward_async(name, request_info, request,
                               timeout, priority, trans_id, pid)
@@ -222,7 +222,7 @@ module CloudI
                     end
                 end
             end
-            send(Erlang.term_to_binary([:forward_sync, name, 
+            send(Erlang.term_to_binary([:forward_sync, name,
                                         OtpErlangBinary.new(request_info),
                                         OtpErlangBinary.new(request),
                                         timeout, priority,
@@ -230,9 +230,9 @@ module CloudI
             raise ForwardSyncException.new()
         end
 
-        def return_(command, name, pattern, response_info, response,
+        def return_(request_type, name, pattern, response_info, response,
                     timeout, trans_id, pid)
-            case command
+            case request_type
             when ASYNC
                 return_async(name, pattern, response_info, response,
                              timeout, trans_id, pid)
@@ -301,7 +301,7 @@ module CloudI
             return poll_request(nil, false)
         end
 
-        def null_response(command, name, pattern, request_info, request,
+        def null_response(request_type, name, pattern, request_info, request,
                           timeout, priority, trans_id, pid)
             return ''
         end
@@ -444,8 +444,17 @@ module CloudI
                         raise TerminateException.new(@timeout_terminate)
                     end
                 when MESSAGE_REINIT
-                    i += j; j = 4
-                    @process_count = data[i, j].unpack('L')[0]
+                    i += j; j = 4 + 4 + 4 + 1 + 1
+                    tmp = data[i, j].unpack("LLLcC")
+                    @process_count = tmp[0]
+                    @timeout_async = tmp[1]
+                    @timeout_sync = tmp[2]
+                    @priority_default = tmp[3]
+                    @request_timeout_adjustment = (tmp[4] != 0)
+                    if @request_timeout_adjustment
+                        @request_timer = Time.now
+                        @request_timeout = 0
+                    end
                     i += j
                 when MESSAGE_KEEPALIVE
                     send(Erlang.term_to_binary(:keepalive))
@@ -487,7 +496,7 @@ module CloudI
             if result[2].length > 0
                 return false
             end
-        
+
             data = recv('')
             data_size = data.bytesize
             if data_size == 0
@@ -608,11 +617,20 @@ module CloudI
                     end
                     API.assert{false}
                 when MESSAGE_REINIT
-                    i += j; j = 4
-                    @process_count = data[i, j].unpack('L')[0]
+                    i += j; j = 4 + 4 + 4 + 1 + 1
+                    tmp = data[i, j].unpack("LLLcC")
+                    @process_count = tmp[0]
+                    @timeout_async = tmp[1]
+                    @timeout_sync = tmp[2]
+                    @priority_default = tmp[3]
+                    @request_timeout_adjustment = (tmp[4] != 0)
+                    if @request_timeout_adjustment
+                        @request_timer = Time.now
+                        @request_timeout = 0
+                    end
                     i += j; j = 4
                     if i == data_size
-                        #
+                        data.clear()
                     elsif i < data_size
                         next
                     else
@@ -622,7 +640,7 @@ module CloudI
                     send(Erlang.term_to_binary(:keepalive))
                     i += j; j = 4
                     if i == data_size
-                        #
+                        data.clear()
                     elsif i < data_size
                         next
                     else
@@ -658,7 +676,7 @@ module CloudI
                 if result[2].length > 0
                     return false
                 end
-    
+
                 data = recv(data)
                 data_size = data.bytesize
                 if data_size == 0
@@ -691,10 +709,6 @@ module CloudI
             return result
         end
 
-        def request_http_qs_parse(request)
-            return text_key_value_parse(request)
-        end
-
         def info_key_value_parse(message_info)
             return text_key_value_parse(message_info)
         end
@@ -717,25 +731,31 @@ module CloudI
             @s.write(data)
         end
 
-        def recv(data)
+        def recv(data_old)
+            data = ''
             if @use_header
-                while data.length < 4
-                    fragment = @s.readpartial(@size)
+                i = 0
+                while i < 4
+                    fragment = @s.readpartial(4 - i)
                     data += fragment
+                    i += fragment.length
                 end
-                total = data[0,4].unpack('N')[0]
-                data.slice!(0..3)
-                while data.length < total
-                    fragment = @s.readpartial(@size)
+                total = data.unpack('N')[0]
+                data = data_old
+                i = 0
+                while i < total
+                    fragment = @s.readpartial([total - i, @size].min)
                     data += fragment
+                    i += fragment.length
                 end
             else
+                data = data_old
                 ready = true
                 while ready == true
                     fragment = @s.readpartial(@size)
                     data += fragment
                     ready = (fragment.bytesize == @size)
-    
+
                     if ready
                         ready = ! IO.select([@s], nil, nil, 0).nil?
                     end
