@@ -9,7 +9,7 @@
 %%%
 %%% BSD LICENSE
 %%% 
-%%% Copyright (c) 2016, Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2016-2017, Michael Truog <mjtruog at gmail dot com>
 %%% All rights reserved.
 %%% 
 %%% Redistribution and use in source and binary forms, with or without
@@ -44,8 +44,8 @@
 %%% DAMAGE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2016 Michael Truog
-%%% @version 0.2.0 {@date} {@time}
+%%% @copyright 2016-2017 Michael Truog
+%%% @version 1.7.0 {@date} {@time}
 %%%------------------------------------------------------------------------
 -module(cloudi_x_syslog_socket).
 -author('mjtruog [at] gmail (dot) com').
@@ -152,7 +152,7 @@ start_link(Options) when is_list(Options) ->
                 {utf8, ?UTF8_DEFAULT},
                 {facility, ?FACILITY_DEFAULT},
                 {app_name, undefined},
-                {path, "/dev/cloudi_x_log"},
+                {path, "/dev/log"},
                 {host, {127,0,0,1}},
                 {port, undefined},
                 {timeout, ?TIMEOUT_DEFAULT}],
@@ -319,8 +319,8 @@ facility(clock0) ->         9; % clock daemon 0
 facility(auth1) ->         10; % security/authorization messages 1
 facility(ftp) ->           11; % FTP daemon
 facility(ntp) ->           12; % NTP subsystem
-facility(auth2) ->         13; % cloudi_x_log audit (security/authorization 2)
-facility(auth3) ->         14; % cloudi_x_log alert (security/authorization 3)
+facility(auth2) ->         13; % log audit (security/authorization 2)
+facility(auth3) ->         14; % log alert (security/authorization 3)
 facility(clock1) ->        15; % clock daemon 1
 facility(local0) ->        16; % local use 0
 facility(local1) ->        17; % local use 1
