@@ -25,9 +25,8 @@ REBAR_DIRS="\
     lib/cloudi_service_udp \
     lib/cloudi_service_validate \
     lib/cloudi_service_zeromq \
-    external/cloudi_x_lager \
-    external/cloudi_x_parse_trans \
     external/cloudi_x_bear \
+    external/cloudi_x_certifi \
     external/cloudi_x_cowboy \
     external/cloudi_x_cowlib \
     external/cloudi_x_elli \
@@ -36,14 +35,21 @@ REBAR_DIRS="\
     external/cloudi_x_epgsql_wg \
     external/cloudi_x_exometer \
     external/cloudi_x_exometer_core \
+    external/cloudi_x_exometer_influxdb \
     external/cloudi_x_folsom \
-    external/cloudi_x_goldrush \
+    external/cloudi_x_hackney \
+    external/cloudi_x_hut \
+    external/cloudi_x_idna \
     external/cloudi_x_jsx \
+    external/cloudi_x_metrics \
+    external/cloudi_x_mimerl \
     external/cloudi_x_msgpack \
     external/cloudi_x_nodefinder \
+    external/cloudi_x_parse_trans \
     external/cloudi_x_pgsql \
     external/cloudi_x_ranch \
     external/cloudi_x_setup \
+    external/cloudi_x_ssl_verify_fun \
     external/proper \
     lib/cgroups \
     lib/cpg \
@@ -91,6 +97,14 @@ case $COMMAND in
             fi
         done
         ;;
+    process)
+        for d in $REBAR_DIRS; do
+            application=`basename $d`
+            # rebar2 creates _drv.so files for no reason
+            rm -f $abs_top_builddir/$d/priv/${application}_drv.so \
+                  $abs_top_builddir/$d/priv/cloudi_x_${application}_drv.so
+        done
+        ;;
     destroy)
         for d in $REBAR_DIRS; do
             application=`basename $d`
